@@ -1,27 +1,27 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
  */
 
-#ifndef __CELL_H_ENV_
-#define __CELL_H_ENV_ 1
+#ifndef OPENAFS_WINNT_AFSD_CELL_H
+#define OPENAFS_WINNT_AFSD_CELL_H 1
 
 #define CELL_MAXNAMELEN                 256
 
 #define CM_CELL_MAGIC    ('C' | 'E' <<8 | 'L'<<16 | 'L'<<24)
 
 /* a cell structure */
-typedef struct cm_cell {        
+typedef struct cm_cell {
     afs_uint32  magic;
     afs_int32 cellID;		        /* cell ID */
     struct cm_cell *allNextp;	        /* locked by cm_cellLock */
     struct cm_cell *nameNextp;	        /* locked by cm_cellLock */
     struct cm_cell *idNextp;	        /* locked by cm_cellLock */
-    struct cm_cell *freeNextp;
+    struct cm_cell *freeNextp;          /* locked by cm_cellLock */
     char name[CELL_MAXNAMELEN];         /* cell name; never changes */
     cm_serverRef_t *vlServersp;         /* locked by cm_serverLock */
     osi_mutex_t mx;			/* mutex locking fields (flags) */
@@ -70,11 +70,20 @@ extern int cm_DumpCells(FILE *, char *, int);
 
 extern void cm_AddCellToNameHashTable(cm_cell_t * cellp);
 
+extern void cm_RemoveCellFromNameHashTable(cm_cell_t *cellp);
+
 extern void cm_AddCellToIDHashTable(cm_cell_t * cellp);
 
-extern long cm_AddCellProc(void *rockp, struct sockaddr_in *addrp, char *namep, 
+extern void cm_RemoveCellFromIDHashTable(cm_cell_t *cellp);
+
+extern long cm_AddCellProc(void *rockp, struct sockaddr_in *addrp, char *namep,
                            unsigned short ipRank);
 
 extern cm_cell_t *cm_UpdateCell(cm_cell_t * cp, afs_uint32 flags);
 
-#endif /* __CELL_H_ENV_ */
+extern long cm_CreateCellWithInfo( char * cellname,
+                                   char * linked_cellname,
+                                   unsigned short vlport, afs_uint32 host_count,
+                                   char *hostname[], afs_uint32 flags);
+
+#endif /* OPENAFS_WINNT_AFSD_CELL_H */
